//+------------------------------------------------------------------+
//|                                                     SignalWZ.mqh |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of Eigen Vectors with PCA v1                       |
//| Type=SignalAdvanced                                              |
//| Name=PCA                                                         |
//| ShortName=PCA                                                    |
//| Class=CSignalPCA                                                 |
//| Page=signal_svm                                                  |
//| Parameter=Price,ENUM_APPLIED_PRICE,PRICE_CLOSE,PCA Applied Price |
//| Parameter=Day,ENUM_DAY_OF_WEEK,THURSDAY,PCA Day of Week          |
//| Parameter=Period,int,30,MA Period                                |
//| Parameter=UsePCA,bool,true,Use PCA                               |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalPCA.                                                |
//| Purpose: Class of trade signals of Eigen Vectors with Principal  |
//| Component Analysis                                               |
//|            Derives from class CExpertSignal.                     |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalPCA                       : public CExpertSignal
{
protected: 

   int                                 m_ma_period;
   
   ENUM_APPLIED_PRICE                  m_price;
   ENUM_DAY_OF_WEEK                    m_day;
   
   bool                                m_pca;
   
   CiMA                                m_MA;
   //--- method of initialization of the indicator
   bool                                InitMA(CIndicators *indicators);

public:

   //--- method of verification of settings
   virtual bool                        ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool                        InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int                         LongCondition(void);
   virtual int                         ShortCondition(void);
   

   void                                Price(ENUM_APPLIED_PRICE value)
   {  m_price = value;
   };

   void                                Day(ENUM_DAY_OF_WEEK value)
   {  m_day = value;
   };

   void                                Period(int value)
   {  m_ma_period = value;
   };

   void                                UsePCA(bool value)
   {  m_pca = value;
   };
   
   MqlDateTime                         __D;

                     CSignalPCA(void);
                    ~CSignalPCA(void);

};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CSignalPCA::CSignalPCA(void)  :   m_ma_period(30)
{
//--- initialization of protected data
   m_used_series = USE_SERIES_OPEN + USE_SERIES_HIGH + USE_SERIES_LOW + USE_SERIES_CLOSE + USE_SERIES_SPREAD + USE_SERIES_TIME;
}
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalPCA::ValidationSettings(void)
{  if(!CExpertSignal::ValidationSettings())
      return(false);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalPCA::InitIndicators(CIndicators *indicators)
{
//--- check pointer
   if(indicators == NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
//--- create and initialize AMA indicator
   if(!InitMA(indicators))
      return(false);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Create MA indicators.                                            |
//+------------------------------------------------------------------+
bool CSignalPCA::InitMA(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- add object to collection
   if(!indicators.Add(GetPointer(m_MA)))
     {
      printf(__FUNCTION__+": error adding MA object");
      return(false);
     }
//--- initialize object
   if(!m_MA.Create(m_symbol.Name(),m_period,m_ma_period,0,MODE_SMA,m_price))
     {
      printf(__FUNCTION__+": error initializing MA object");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalPCA::LongCondition(void)
{  int _result = 0;
   m_MA.Refresh(-1);
   m_close.Refresh(-1);
   m_time.Refresh(-1);
//
   if(m_MA.Main(StartIndex()+1) > m_close.GetData(StartIndex()+1) && m_MA.Main(StartIndex()) < m_close.GetData(StartIndex()))
   {  _result = 100;
      //PrintFormat(__FUNCSIG__);
   }
   if(m_pca)
   {  TimeToStruct(m_time.GetData(StartIndex()),__D);
      if(__D.day_of_week != m_day)
      {  _result = 0;
      }
   }
//
   return(_result);
}
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalPCA::ShortCondition(void)
{  int _result = 0;
   m_MA.Refresh(-1);
   m_close.Refresh(-1);
//
   if(m_MA.Main(StartIndex()+1) < m_close.GetData(StartIndex()+1) && m_MA.Main(StartIndex()) > m_close.GetData(StartIndex()))
   {  _result = 100;
      //PrintFormat(__FUNCSIG__);
   }
   if(m_pca)
   {  TimeToStruct(m_time.GetData(StartIndex()),__D);
      if(__D.day_of_week != m_day)
      {  _result = 0;
      }
   }
//
   return(_result);
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
void CSignalPCA::~CSignalPCA(void)
{
}
//+------------------------------------------------------------------+
